@@ -1,3 +1,5 @@
+import gzip
+import json
 import logging
 
 from typing import Optional, Mapping, Callable, Any, Tuple, List, Type, Iterable, Dict
@@ -14,11 +16,12 @@ from flask import (
     Response as FlaskResponse,
 )
 from werkzeug.datastructures import Headers
+from werkzeug.routing import Rule, parse_converter_args
 
 from .config import Config
 from .page import PAGES
 from .types import ResponseBase, RequestBase
-from .utils import parse_multi_dict
+from .utils import parse_multi_dict, parse_rule
 
 
 @dataclass
@@ -54,13 +57,12 @@ class FlaskBackend:
         for method in route.methods:
             yield method, func
 
-    def parse_path(self, route: Any) -> Tuple[str, List[Any]]:
-        from werkzeug.routing import parse_rule, parse_converter_args
+    def parse_path(self, route: Rule) -> Tuple[str, List[Any]]:
 
         subs = []
         parameters = []
 
-        for converter, arguments, variable in parse_rule(str(route)):
+        for converter, arguments, variable in parse_rule(route):
             if converter is None:
                 subs.append(variable)
                 continue
@@ -139,8 +141,16 @@ class FlaskBackend:
             req_query = parse_multi_dict(raw_query)
         else:
             req_query = {}
-        if request.content_type == "application/json":
-            parsed_body = request.get_json() or {}
+        if request.content_type and "application/json" in request.content_type:
+            if request.content_encoding and "gzip" in request.content_encoding:
+                raw_body = gzip.decompress(request.stream.read()).decode(
+                    encoding="utf-8"
+                )
+                parsed_body = json.loads(raw_body)
+            else:
+                parsed_body = request.get_json() or {}
+        elif request.content_type and "multipart/form-data" in request.content_type:
+            parsed_body = parse_multi_dict(request.form) if request.form else {}
         else:
             parsed_body = request.get_data() or {}
         req_headers: Optional[Headers] = request.headers or None
